// Cache.h: interface for the Cache class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(_CACHE_H_INCLUDED_)
#define _CACHE_H_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "global.h"
#include "Page.h"
#include "Hash.h"

typedef ListPage* PListPage;
typedef vector<PListPage> VecPListPage;

// class Cache stores all blocks of the cache
// this is an abstract class, which should be subclassed for
// the algorithms LRU, LFU, and random
//
// the data structure of the cache is a array of doubly linked list m_vecPLsPgCache
// blocks (pages) with different reference count are placed on 
// different list. In LRU and random, all blocks have reference
// count 1, therefore, all pages are stored on the list m_vecPLsPgCache[1]
class Cache  
{
protected: // protected members
  void inc(int idx, int n=1);
  virtual void add(PPage pPage);
  virtual void evict()=0; // evict a page
  virtual void updatePage(ItListHashNode cur) {};
  virtual void erase(PPage pPage);
  void eraseFile(PListPage pListPages, PPage pPage);

  virtual void LogStats(PEvent pEvent);
  virtual void LogHit(PEvent pEvent);
  virtual void LogMiss(PEvent pEvent);

  int GetDepth(ItListPage itPage); // get the stack depth
protected:
  void check();
  VecPListPage m_vecPLsPgCache; // cache
  DWORD m_iPageNum; // number of pages in cache
  BOOL m_bWarmuping; // whether or not now is warmuping
  Hash m_hash; // the hash table for the cache
  char m_sName[10];
  ItListPage m_itCur; // current hit page
  int m_iEvtNum;

public: // public members
  int m_iAlg;
  int m_iSize; // cache size
  VecInt m_vecStats; // statistics variables
  virtual void print(ofstream &of);
  virtual BOOL propagate(PEvent pEvent);
  void SetWarmup(BOOL warmup) { m_bWarmuping = warmup; };
public:
  Cache(int iSize);
  virtual ~Cache();
};

typedef Cache* PCache;

#endif // !defined(_CACHE_H_INCLUDED_)
