// CacheLFUStackDepth.cpp: implementation of the CacheLFUStackDepth class.
//
//////////////////////////////////////////////////////////////////////

#include "global.h"
#include "CacheLFUStackDepth.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CacheLFUStackDepth::CacheLFUStackDepth(int iSize, char *sOutName)
  :CacheLFU(iSize)
{
  m_iSize = INT_MAX;
  m_sOutName = sOutName;
}

CacheLFUStackDepth::~CacheLFUStackDepth()
{

}

//////////////////////////////////////////////////////////////////////
// Protected methods
//////////////////////////////////////////////////////////////////////

// log the stack depth of hit
void CacheLFUStackDepth::LogHit(PEvent pEvent)
{
  int depth;

  CacheLFU::LogHit(pEvent);
  depth = GetDepth(m_itCur);
  if (depth+1 > m_vecSDepth.size())
    m_vecSDepth.resize(depth+1,0);
  m_vecSDepth[depth]++;
}

//////////////////////////////////////////////////////////////////////
// Public methods
//////////////////////////////////////////////////////////////////////

// print the stack depth
void CacheLFUStackDepth::print(ofstream &)
{
  int i, size, sum, cdf;
  string sPdfName, sCdfName;
  ofstream ofPdf, ofCdf;

  sPdfName = m_sOutName+"pdf.txt";
  ofPdf.open(sPdfName.c_str());
  if (!ofPdf)
    Error("Error open %s",sPdfName.c_str());

  sum = 0;
  size = m_vecSDepth.size();
  for (i=0; i<size; i++)
    if (m_vecSDepth[i]!=0)
    {
      sum += m_vecSDepth[i];
      ofPdf << i << "\t" << m_vecSDepth[i] << endl;
    }

  ofPdf.close();

  sCdfName = m_sOutName+"cdf.txt";
  ofCdf.open(sCdfName.c_str());
  if (!ofCdf)
    Error("Error open %s",sCdfName.c_str());

  cdf = 0;
  for (i=0; i<size; i++)
    if (m_vecSDepth[i]!=0)
    {
      cdf += m_vecSDepth[i];
      ofCdf << i << "\t" << cdf*1.0/sum << endl;
    }

  ofCdf.close();
}
