// Experiment.cpp: implementation of the Experiment class.
//
//////////////////////////////////////////////////////////////////////

#include "global.h"
#include "Experiment.h"
#include "CacheLRU.h"
#include "CacheLFU.h"
#include "CacheRand.h"
#include "CacheLRUStackDepth.h"
#include "CacheLFUStackDepth.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Experiment::Experiment(int iCltSize, int iWarmup, 
                       int iSvrSize, int iAlg, int iSeed)
{
  m_iWarmup = iWarmup;
  m_iEvtNum = 0;
  pClient = new CacheLRU(iCltSize);
  switch (iAlg)
  {
  case LRU:
    pServer = new CacheLRU(iSvrSize);
    break;
  case LFU:
    pServer = new CacheLFU(iSvrSize);
    break;
  case RAND:
    pServer = new CacheRand(iSvrSize, iSeed);
    break;
  case LRUDEP:
    pServer = new CacheLRUStackDepth(iSvrSize, "lrud");
    break;
  case LFUDEP:
    pServer = new CacheLFUStackDepth(iSvrSize, "lfud");
    break;
  default:
    assert(FALSE);
    break;
  }
}

Experiment::~Experiment()
{
  delete pClient;
  delete pServer;
}

//////////////////////////////////////////////////////////////////////
// public methods
//////////////////////////////////////////////////////////////////////

// process pEvent for the client cache, and server cache if it is 
// propagated by the client cache
void Experiment::ProcessEvent(PEvent pEvent)
{
  m_iEvtNum++;
  if (m_iEvtNum>=m_iWarmup)
  {
    pClient->SetWarmup(FALSE);
    pServer->SetWarmup(FALSE);
  }
  if (pClient->propagate(pEvent))
    pServer->propagate(pEvent);
}

// print out statistic data for the client and server
void Experiment::print(ofstream &ofOutput)
{
  int i;
  ofOutput << "Name Alg Size  ";
  for (i=0; i<End; i++)
    ofOutput << name[i];
  ofOutput << endl;
  ofOutput << "Clt ";
  pClient->print(ofOutput);
  ofOutput << "Svr ";
  pServer->print(ofOutput);
  ofOutput << endl;
}

VecInt& Experiment::GetStats()
{
  return pServer->m_vecStats;
}
