/*
 * This program reads a (binary) trace file and prints out the
 * contents of each trace event in text to stdout
*/


#include <stdio.h>
#include <stdlib.h>
#include <memory.h>

/*
// trace event structure
struct event {
  int rw;                                                 // 0=read, 1=write 
  unsigned long dev;                                        // device number 
  unsigned long inode;                                       // inode number 
  unsigned long block;                                       // block number 
};
*/

/* node structure used to hold a single reference when read from */
/* a trace file */
#define WRITE 0
#define READ  1
#define UNKNOWN 2
struct event {
  unsigned long time;     /* time of event in milli_sec since start of trace */
  int dev;
  int inode;
  int block;
  int flag;               /*  0 = non-shared file ref (we set this to the
                client # immediately after reading),
           -1 = shared file ref,
           -2 = unlink (vnsremove) event */
  int rw;                 /* 0 = write, 1 = read, 2 = unknown */
};


/* given a file pointer, return a pointer to the next trace event */
/* if end of file is reached or an error occurs, return NULL */

struct event *get_next_event(FILE *infile)
{
  struct event *evnt;
  char small_endian[24], big_endian[24];
  int i;

  if((evnt = (struct event *)malloc(sizeof(struct event))) == 0) {
    fprintf(stderr, "*** Error mallocing new event record ***\n");
    exit(0);
  }
  if(!fread(small_endian, sizeof(struct event), 1, infile)) {
    free(evnt);
    evnt = NULL;
    return evnt;
  }

  for (i=0; i<sizeof(event)/sizeof(long); i++)
  {
    big_endian[i*4+0] = small_endian[i*4+3];
    big_endian[i*4+1] = small_endian[i*4+2];
    big_endian[i*4+2] = small_endian[i*4+1];
    big_endian[i*4+3] = small_endian[i*4+0];
  }  
  memcpy(evnt,big_endian,sizeof(struct event));
  return(evnt);
}


void main(int argc, char *argv[])
{
  FILE *infile;                                            /* the trace file */
  struct event *current_event;       /* pointer to the last read trace event */
  int done = 0;                    /* set when end of trace has been reached */
 
  /* check args */
  if(argc != 2) { 
    fprintf(stderr, "Usage: %s <trace_file>\n", argv[0]);
    exit(0);
  }

  /* open trace file */
  if((infile = fopen(argv[1], "rb")) == 0) {
    fprintf(stderr, "*** Error -- can not open trace file %s ***\n", argv[1]);
    exit(0);
  }

  /* main loop */
  while(!done) {

    /* read next event */
    current_event = get_next_event(infile);

    if(current_event == NULL)
      done = 1;
    else {
      /* do something interesting */
      if(current_event->rw == READ)
        printf("read ");
      else if (current_event->rw == WRITE)
        printf("write ");
      else if (current_event->rw == -1)
        printf("erase ");
      else
        printf ("unknown(%d) ", current_event->rw);
     // printf("dev= %u inode= %u block= %u\n",
     // current_event->dev, current_event->inode, current_event->block);
      printf("dev= %d inode= %d block= %d time= %u  flag=%d\n",
      current_event->dev, current_event->inode, current_event->block,
      current_event->time, current_event->flag);
      //printf("dev= %X inode= %X block= %X\n",
      //current_event->dev, current_event->inode, current_event->block);
    
      /* dispose of current event */
      free(current_event);
    }
    }
}
