// Experiment.h: interface for the Experiment class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(_GLOBAL_H_INCLUDED_)
#define _GLOBAL_H_INCLUDED_

#if _MSC_VER > 1000
#pragma once

// The debugger can't handle symbols more than 255 characters long.
// STL often creates symbols longer than that.
// When symbols are longer than 255 characters, the warning is issued.
#pragma warning(disable:4786)

#endif // _MSC_VER > 1000

#include <time.h>

#include <cstdio>
#include <cstdlib>
#include <climits>
#include <cfloat>
#include <cassert>
#include <cstdarg>
#include <strstream>
#include <iostream>
#include <fstream>

#include <algorithm>
#include <vector>
#include <list>
#include <string>

using namespace std;

typedef unsigned long DWORD;

/* trace event structure */
/* node structure used to hold a single reference when read from */
/* a trace file */
typedef struct Event {
  unsigned long time;     /* time of event in milli_sec since start of trace */
  int dev;
  int inode;
  int block;
  int flag;               /*  0 = non-shared file ref (we set this to the
                client # immediately after reading),
           -1 = shared file ref,
           -2 = unlink (vnsremove) event */
  int rw;                 /* 0 = write, 1 = read, 2 = unknown */
}* PEvent;

typedef vector<int> VecInt;

typedef list<PEvent> ListPEvent;
typedef ListPEvent::iterator ListItPEvent;

#define WRITE 0
#define READ  1
#define ERASE -1

// supported algorithms
enum ALGORITHM {
  LRU,
  RAND,
  LFU,
  LRUDEP,
  LFUDEP
};

// variables for statistics
enum
{
  Num    ,
  Read   ,
  Write  ,
  Erase  ,
  Hit    ,
  Miss   ,
  RdHit  ,
  RdMiss ,
  WtHit  ,
  WtMiss ,
  ErHit  ,
  ErMiss ,
  End       
};

// names of the statistics variables
extern char name[][8];

typedef int BOOL;

#define FALSE   0
#define TRUE    1

#define verify(exp) if((exp));else Error("Error %s in %s(%d)",#exp, __FILE__, __LINE__)

void Error(char *fmt, ...);

#endif // !defined(_GLOBAL_H_INCLUDED_)
