// main.cpp : the main program
#include "global.h"
#include "Experiment.h"
#include "Config.h"
#include "Simulation.h"

// name of the statistics variables
char name[][8]=
{
  "Num    ",
  "Read   ",
  "Write  ",
  "Erase  ",
  "Hit    ",
  "Miss   ",
  "RdHit  ",
  "RdMiss ",
  "WtHit  ",
  "WtMiss ",
  "ErHit  ",
  "ErMiss ",
  "End    "
};

// print error message and exit
void Error(char *fmt, ...)
{
  va_list args;

  cerr << "Error: ";

  va_start(args, fmt);
  vfprintf(stderr, fmt, args);
  fprintf(stderr, "\n");
  va_end(args);
    
  exit(1);
}

//For Visual C++
#ifdef  _MSC_VER
#include <crtdbg.h>  //detect memory check
//Check memory leak when in Visual C++
void EnableDebugNew()
{
  // Get current flag
  int tmpFlag = _CrtSetDbgFlag( _CRTDBG_REPORT_FLAG );

  // Turn on leak-checking bit
  tmpFlag |= _CRTDBG_LEAK_CHECK_DF;

  // Turn off CRT block checking bit
  tmpFlag &= ~_CRTDBG_CHECK_CRT_DF;

  // Set flag to the new value
  _CrtSetDbgFlag( tmpFlag );
}
#endif

int main(int argc, char *argv[])
{
#ifdef _MSC_VER
  //EnableDebugNew();
#endif

  if (argc != 3)
  {
    Error("Usage: cltsrv <config_file> <output_file>");
  }

  char *strConfigName, *strOutputName;
  Simulation sim;

  strConfigName = argv[1];
  strOutputName = argv[2];

  sim.ReadConfig(strConfigName);

  sim.run();

  sim.print(strOutputName);

  // print the running time
  int t = clock()/CLOCKS_PER_SEC;
  cerr << "Run time: " << t/60 << " min " << t%60 << " sec  " << endl;
  cerr << '\a'; // beep

  return 0;
}